<?php

include_once(dirname(__FILE__).'/widget-multichoice.php');

class WidgetList extends WidgetMultiChoice {

    var $itemOptGroup;      // indexed by optgroup and list names in group
    var $allowNone = false;
    var $groups;

    var $allowOther  = false;
    var $otherValue  = NULL;
    var $otherDesc   = NULL;
    var $otherWidget = NULL;
    var $otherData   = NULL;

    // Constructor
    function WidgetList($name, $htmlid=NULL, $form_id=NULL, $dbid=NULL) {
        parent::WidgetMultiChoice($name, $htmlid, $form_id, $dbid);
        $this->itemOptGroup = array( NULL=>array() );
        $this->groups       = array();
        $this->allowNone    = false;
    }

    function addItem($name, $description=NULL, $optgroup=NULL) {
        parent::addItem($name, $description);
        if(!isset($this->itemOptGroup[$optgroup]))
            $this->itemOptGroup[$optgroup] = array();
        $this->itemOptGroup[$optgroup][] = $name;
    }

    function addGroup($name, $description) {
        $this->groups[$name] = $description;
    }

    function addOtherChoice($w, $desc='Other ...', $val='**OTHER**'){
        assert(!is_null($w->htmlid));
        assert(!is_null($this->htmlid));
        $this->allowOther  = true;
        $this->otherValue  = $val;
        $this->otherDesc   = $desc;
        $this->otherWidget = $w;
        $this->otherData   = NULL;
        $this->afterLabel  = array($this, '_htmlCodeAfterlabel');
        $this->addItem($val, $desc);
    }

    function _htmlCodeAfterlabel(){
        return $this->otherWidget->htmlCode().
            "<script type=\"text/javascript\">document.getElementById('".addslashes($this->htmlid)."').onchange();</script>";
    }

    // Return HTML code
    function htmlCode() {
        $res = '<select name="'.htmlspecialchars($this->name).'"';
        if(!is_null($this->htmlid)){
            $res .= ' id="'.htmlspecialchars($this->htmlid).'"';
        }
        if($this->allowOther){
            $jscode = "document.getElementById('".addslashes($this->otherWidget->htmlid)."').className = (this.value == '".addslashes($this->otherValue)."') ? '' : 'hidden';";
            $res .= ' onChange="'.htmlspecialchars($jscode).'"';
        }
        $res .= ">\n";
        $res .= "\t<!-- default value: ".htmlspecialchars($this->defaultValue)." -->\n";
        foreach($this->itemOptGroup as $group=>$names){
            if($group){
                $gname = isset($this->groups[$group]) ? $this->groups[$group] : $group;
                $res .= "\t<!-- group: ".htmlspecialchars(var_export($group, true))." -->\n";
                $res .= "\t<optgroup label=\"".htmlspecialchars($gname)."\">\n";
            }
            foreach($names as $name){
                $desc = $this->itemDescriptions[$name];
                $res .= "\t  <option value=\"".htmlspecialchars($name).'"';
                if($this->defaultValue === $name){
                    $res .= ' selected="selected"';
                }
                $res .= '>'.htmlspecialchars($desc)."</option>\n";
            }
            if($group){
                $res .= "\t</optgroup>\n";
            }
        }
        $res .= "</select>";
        //if($this->allowOther){
        //    $res .= $this->otherWidget->htmlCode();
        //}
        return $res;
    }

    // Validate GET/POST variable
    // Return NULL if validation fails. In that case it set
    // $this->validationError to a message explaining why it failed.
    // On success it returns the converted data and $this->validationError is
    // cleared to NULL
    function validate($data){
        if($this->allowOther && $data == $this->otherValue) {
            if(!is_null($this->otherData)){
                $res = $this->otherWidget->validate($this->otherData);
                $this->validationError = $this->otherWidget->validationError;
                return $res;
            } else {
                $this->validationError = "No other value to validate";
                return NULL;
            }
        } elseif(!in_array($data, $this->itemNames, true)) {
            $this->validationError = "Value '$data' not in range";
            return NULL;
        } else {
            $this->validationError = NULL;
            return $data;
        }
    }

    // Return true if the data is other.
    function isOther($data){
        return $this->allowOther && $data == $this->otherValue;
    }

    // insert in database using $ARR[$this->name] as data. Behaves the same way
    // as insertDatabase()
    // This will also change the default value to the new value inserted.
    function insertDatabaseFrom($ARR, $newDefault=true){
        if($this->isOther($ARR[$this->name])){
            $this->otherData = $ARR[$this->otherWidget->name];
            //if($newDefault){
            //    $this->defaultValue = $this->otherValue;
            //    $newDefault=false;
            //}
        }
        if(isset($ARR[$this->name])){
            return $this->insertDatabase($ARR[$this->name], $newDefault);
        } else {
            return $this->insertDatabase(NULL, $newDefault);
        }
    }

    // Set the default value
    function setDefaultValue($val=NULL){
        if(is_null($val)) $val = $this->getCurrentValue();
        if($this->allowOther && !in_array($val, $this->itemNames, true)){
            // if not in range, activate other and set default value for the
            // other widget
            $this->defaultValue = $this->otherValue;
            $this->otherWidget->setDefaultValue($val);
        } else {
            $this->defaultValue = $val;
            if($this->allowOther && $val == $this->otherValue && !is_null($this->otherData)) {
                // if other is activated, tell the other widget to set its
                // default value to the one we have
                $this->otherWidget->setDefaultValue($this->otherData);
            }
        }
    }


}

?>