<?php

include_once(dirname(__FILE__).'/common.php');
include_once(dirname(__FILE__).'/bdd-insert.php');
include_once(dirname(__FILE__).'/bdd-get.php');
include_once(dirname(__FILE__).'/label.php');

class Widget {

    var $name   = NULL;
    var $dbid   = NULL;
    var $htmlid = NULL;

    var $defaultValue = NULL;

    var $funcBddInsert = 'bdd_insert';
    var $funcBddGet    = 'bdd_get';
    var $form_id = NULL;

    var $validationError = NULL;
    var $databaseError   = NULL;

    // true if in the htmlcode, the <label> tag is used.
    var $usesLabel = false;
    var $afterLabel = NULL; // method to call after </label> called with call_user_func()

    // Constructor
    function Widget($name, $htmlid=NULL, $form_id=NULL, $dbid=NULL) {
        if(is_null($dbid)) $dbid=$name;
        $this->name = $name;
        $this->dbid = $dbid;
        $this->htmlid = $htmlid;
        $this->form_id = $form_id;
        if(!is_null($form_id)){
            $this->setDefaultValue();
        }
        $this->usesLabel = false;
    }

    // Creates a label for the widget
    function createLabel($desc, $class=NULL){
        return new Label($this, $desc, $class);
    }

    // Return HTML code
    function htmlCode() {
        return "<!-- deferred -->";
    }

    // Return HTML code for validation error if set
    function htmlValidationError() {
        if(!is_null($this->validationError)){
            return '<span class="error">'
                    .htmlspecialchars($this->validationError)
                    .'</span>';
        } else {
            return '';
        }
    }

    function htmlAllCode(){
        $res  = $this->htmlCode();
        $res .= " ";
        $res .= $this->htmlValidationError();
        return $res;
    }

    // print HTML code and validation error if there is
    function printHtmlCode() {
        print $this->htmlAllCode();
        print "\n";
    }

    // Validate GET/POST variable
    // Return NULL if validation fails. In that case it set
    // $this->validationError to a message explaining why it failed.
    // On success it returns the converted data and $this->validationError is
    // cleared to NULL
    function validate($data){
        $this->validationError = NULL;
        return $data;
    }

    // Insert the data $val after validating it
    // If validation fails, $this->validationError is set and false is returned
    // If database insertion, $this->databaseError is set and the error is
    // thrown
    // On success returns true
    function insertDatabase($data, $newDefault=false, $validate=true){
        assert(!is_null($this->form_id));
        if($validate) $data = $this->validate($data);
        if($newDefault) $this->setDefaultValue($data);
        if(is_null($data)){
            //throw $this->validationError;
            return false;
        }
        $this->databaseError = NULL;
        $data = $this->serializeData($data);
        $funcBddInsert = $this->funcBddInsert;
        $funcBddInsert($this->form_id, $this->dbid, $data, $this->databaseError);
        if(!is_null($this->databaseError)){
            throw $this->databaseError;
            return false;
        }
        return true;
    }

    // insert in database using $ARR[$this->name] as data. Behaves the same way
    // as insertDatabase()
    // This will also change the default value to the new value inserted.
    function insertDatabaseFrom($ARR, $newDefault=true){
        $t = microtime(true);
        if(isset($ARR[$this->name]))
            $res = $this->insertDatabase($ARR[$this->name], $newDefault);
        else
            $res = $this->insertDatabase(NULL, $newDefault);
        dbg("%s->insertDatabaseFrom(): %gs\n", $this->name, microtime(true)-$t);
        return $res;
    }

    function validateFrom($ARR){
        return $this->validate(isset($ARR[$this->name])?$ARR[$this->name]:NULL);
    }

    // Get the current value from the database and return it on success, or
    // raise the error message
    function getCurrentValue(){
        assert(!is_null($this->form_id));
        $this->databaseError = NULL;
        $funcBddGet = $this->funcBddGet;
        $val = $funcBddGet($this->form_id, $this->dbid, $this->databaseError);
        if(!is_null($this->databaseError)){
            throw $this->databaseError;
            return false;
        }
        return $this->unserializeData($val);
    }

    // Set the default value
    function setDefaultValue($val=NULL){
        $this->defaultValue = is_null($val) ? $this->getCurrentValue() : $val;
    }

    // Prepare the data to be stored in the database
    function serializeData($data){
        return $data;
    }

    // Inverse of serializeData()
    function unserializeData($data){
        return $data;
    }

}

?>